unit UnitMain;

interface

uses
  System.SysUtils, System.Classes, JS, Web,

  generics.Collections,

  WEBLib.Graphics, WEBLib.Controls,
  WEBLib.Forms, WEBLib.Dialogs, Vcl.StdCtrls, WEBLib.StdCtrls, Vcl.Controls,
  WEBLib.WebCtrls,

  Unit1,
  Unit2;

type
  TFormList = class(TObjectDictionary<string, TWebHTMLDiv>);

  TMainForm = class(TWebForm)
    TitleDiv: TWebHTMLDiv;
    WebButton1: TWebButton;
    WebButton2: TWebButton;
    FormsDiv: TWebHTMLDiv;
    WebLabel1: TWebLabel;
    procedure WebButton1Click(Sender: TObject);
    procedure WebFormCreate(Sender: TObject);
    procedure WebButton2Click(Sender: TObject);
  private
    { Private declarations }
    FFormList: TFormList;
    FCurrentFormDiv: TWebHTMLDiv;
    function CreateForm(cls: TFormClass; const parentDivId, formName: string; clr: TColor): TWebHTMLDiv;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

{ TForm11 }

//------------------------------------------------------------------------------
function TMainForm.CreateForm(cls: TFormClass; const parentDivId,
  formName: string; clr: TColor): TWebHTMLDiv;
var
  //formParentDiv: TWebHTMLDiv;
  frm: TWebForm;
  formCount: integer;
begin
  // -- Create the <div> that will hold the form. This div should be a child
  // -- of 'forms_div'.
  result := TWebHTMLDiv.Create('forms_div');
  result.ElementPosition := epAbsolute;

  asm
    var frmDiv = document.getElementById("forms_div");
    console.log('frmDiv.childElementCount before = ' + frmDiv.childElementCount);
  end;

  result.ElementID := parentDivId;

  // -- For "Form2", i.e. the second call to CreateForm, the childElementCount
  // -- is equal to zero at this point because the "form1_parent" div appears to
  // -- become a sibling of "forms_div" rather than a child.
  asm
    var frmDiv = document.getElementById("forms_div");
    console.log('frmDiv.childElementCount after = ' + frmDiv.childElementCount);
  end;

  // -- With multiple forms the following results in a form hierarchy for Form1
  // -- and Form2 rather than creating siblings. Therefore, try using Javascript
  // -- but it fails because we can't get the correct childElementCount.
  //result.Parent := FormsDiv;

  result.ElementHandle.style.setProperty('width', '100%');
  result.ElementHandle.style.setProperty('height', '100%');

  asm
    var cntnr = document.getElementById(parentDivId);
    var frmDiv = document.getElementById("forms_div");

    if(frmDiv.childElementCount > 0){
      frmDiv.firstChild.insertAdjacentElement('beforebegin', cntnr);
    }else{
      frmDiv.appendChild(cntnr);
    }

    setTimeout(() => { window.dispatchEvent(new Event('resize')); }, 0);
  end;

  Application.CreateFormDirect(cls, frm);
  frm.FormContainer := result.ElementID;
  frm.Align := alClient;
  frm.Color := clr;

  FFormList.Add(formName, result);
  FCurrentFormDiv := result;
end;
//------------------------------------------------------------------------------
procedure TMainForm.WebButton1Click(Sender: TObject);
var
  formDiv: TWebHTMLDiv;
begin
  if assigned(FCurrentFormDiv) then
    FCurrentFormDiv.Visible := false;

  if not FFormList.TryGetValue('Form1', formDiv) then
    FCurrentFormDiv := CreateForm(TForm1, 'form1_parent', 'Form1', clYellow)
  else
  begin
    formDiv.Visible := true;
    FCurrentFormDiv := formDiv;
  end;
end;
//------------------------------------------------------------------------------
procedure TMainForm.WebButton2Click(Sender: TObject);
var
  formDiv: TWebHTMLDiv;
begin
  if assigned(FCurrentFormDiv) then
    FCurrentFormDiv.Visible := false;

  if not FFormList.TryGetValue('Form2', formDiv) then
    FCurrentFormDiv := CreateForm(TForm2, 'form2_parent', 'Form2', clBlue)
  else
  begin
    formDiv.Visible := true;
    FCurrentFormDiv := formDiv;
  end;
end;
//------------------------------------------------------------------------------
procedure TMainForm.WebFormCreate(Sender: TObject);
begin
  FFormList := TFormList.Create{([doOwnsValues])};
end;

end.